/*
	NRob Library
		CPU　ATMEGA328P
			motor Driver:TB6612 D2-D7
			Servo	D8,D9
			超音波センサ	D10-D13
			Analog 6ケ　A0
		I2C Expander MVP23017
			LCD 16*2
			SW 5ケ
v1.0	20210410	ライブラリ化
v1.1	20211106	loop動作見直し サーボ動作＋LED＋LCD
*/

#include "NRob.h"

//---	割込みタイマー用	-----
byte	flg10ms=0;

//*** タイマ割込み    **************************************
ISR (TIMER2_COMPA_vect)			//割込み　2ms
{
	static int	cnt10ms=0;

  if(++cnt10ms>=5){
		cnt10ms=0;
		flg10ms=1;									//10ms
  }
}

//---------------------------------------------------------
/*
// NRob基板コンストラクタ
NRob3::NRob3(void)
{
}
*/
//*** タイマ動作    ****************************************
//------------------------------------------------
bool  NRob3::loop(void)									//基本ループ 50ms毎true
{
	bool ans=false;
	static int	cnt100=0;

	chk_AD();															//AD入力ループ
	if(flg10ms){
		flg10ms=0;
		chk_SW();
//		dsp_Loop();													//LCD表示ループ対応

  	if(++cntUsr>=usrTime){							//指定ピッチ
			cntUsr=0;
			chk_US();													//超音波チェック
			read_Compass();										//コンパス読み込み動作
			read_SubIn();											//カラーセンサ読込 
			ans = true;
		}
		if(++cnt100>=10){										//100ms
			cnt100=0;
			LED_send();												//LED定期送信
			//act_ServoLoop();
		}
	}
	return ans;
}
//------------------------------------------------
void  NRob3::init_Tm2(void)							//Timer2動作設定
{
  TCCR2A  = 0;           
  TCCR2B  = 0;          
  TCCR2A |= (1 << WGM21);         			//CTC mode
  TCCR2B |= 0x05;//128分周　0x04;	//分周64
  OCR2A   = 250-1;		         			//250カウントごと->16ms
  TIMSK2 |= (1 << OCIE2A);
}
//------------------------------------------------
//*** 初期化    ********************************************
// NRob基板の初期化
void	NRob3::init(int pich)							//loopでオンするピッチ(ms)
{
  Wire.begin();       

  pinMode( MTA1, OUTPUT );  //すべて出力
  pinMode( MTA2, OUTPUT );
  pinMode( MTAP, OUTPUT );
  pinMode( MTB1, OUTPUT );
  pinMode( MTB2, OUTPUT );
  pinMode( MTBP, OUTPUT );
	mpowA=0;
	mpowB=0;

  pinMode(S7TRIG, OUTPUT);        			//超音波センサ
  pinMode(S7ECHO, INPUT);
  pinMode(S8TRIG, OUTPUT);
  pinMode(S8ECHO, INPUT);

  for(int i=0; i<5; i++){               //SW変数初期化
    inSW[i]=0;
    tmSW[i]=0;
  }
	read_parK();													//parK読込
																				//MPC23017 init
	read_ofset();													//コンパスオフセット読込

	I2Cadr = MCP23017_ADR;
	I2Cbuf[0]=0x00;	I2Cbuf[1]=0x1F;				//GPA設定 bit4-0 IN
  I2C_Send(2); 
	I2Cbuf[0]=0x01;	I2Cbuf[1]=0x00;				//GPB設定 ALL OUT
  I2C_Send(2); 
	I2Cbuf[0]=0x0C;	I2Cbuf[1]=0x1F;				//GPA プルアップ設定
  I2C_Send(2); 

	LCD_init();

	if(chk_Compass()){								//コンパスあれば初期化
		init_Compass();
	}

	delay(50);
	usrTime = pich/10;										//指定ピッチ
	init_Tm2();														//タイマ割込み設定

	LED_AllOFF();

	chk_SW();															//Sel押し電源入りは特殊モード
	if(!(inGPAnow&0x01))	Special_Mode();

}

//------------------------------------------------


//*** AD入力    *********************************************

//------------------------------------------------
bool  NRob3::chk_AD(void)						//ADを順に取込 繰返しループでコール
{
  byte low, high;
  int tmp;
	bool ans=false;

  if(bit_is_clear(ADCSRA, ADSC)){ //AD変換終了で次へ
    low  = ADCL;
    high = ADCH;
    tmp = (high << 8) | low;          //ADをセット
		adAdd[adCnt] += tmp;								//平均累計

    if(++adCnt>5){             					//次を起動
    	adCnt=0; 
			ans=true;

			aveCnt++;													//平均
			if(aveCnt>=5){
			  for(int i=0; i<6; i++){
					ave[i] = adAdd[i]/5;
					adAdd[i]=0;
				}
				aveCnt=0;
			}
		}
    ADMUX = (0x40) | (adPin[adCnt] & 0x07);
    _SFR_BYTE(ADCSRA) |= _BV(ADSC);
  }
	return ans;
}
//------------------------------------------------
int  NRob3::get_AD(byte no)				//AD値を返す 1～6番
{
	if(no<=6)	return ave[no-1];
	else			return 0;
}

//*** TB6612 操作     ************************************************

//---------------------------------------------------------
// モータの駆動　AB両方
void	NRob3::act_motorAB(int pa, int pb)
{
	act_motorA(pa);
	act_motorB(pb);
}
//---------------------------------------------------------
//モーターA駆動
															//MTA1=PD2 /MTA2=PD3/ MTAP=PD5
void	NRob3::act_motorA(int pa)
{
  if(mpowA==pa) return;

  PORTD |= _BV(2);
  PORTD &= ~_BV(2);  

  if(pa==0){                  //ストップ
    PORTD &= ~_BV(2);					// PD2=LOW
		PORTD &= ~_BV(3);					// PD3=LOW
    analogWrite(MTAP, 0);
  }else if(pa>0){             //正転
		PORTD |= _BV(2);					//	PD2=HIGH
		PORTD &= ~_BV(3);					//	PD3=LOW
    analogWrite(MTAP, pa);
  }else{                      //逆転
		PORTD &= ~_BV(2);					//	PD2=LOW
		PORTD |= _BV(3);					//	PD3=HIGH
    analogWrite(MTAP, -pa);
  }
  mpowA=pa;
}
//---------------------------------------------------------
//モーターB駆動
															//MTB1=PD4 /MTB2=PD7/ MTBP=PD6
void	NRob3::act_motorB(int pb)
{
  if(mpowB==pb) return;
  
  if(pb==0){                  //ストップ
    PORTD &= ~_BV(4);					// PD4=LOW
		PORTD &= ~_BV(7);					// PD7=LOW
    analogWrite(MTBP, 0);
  }else if(pb>0){             //正転
		PORTD |= _BV(4);					//	PD4=HIGH
		PORTD &= ~_BV(7);					//	PD7=LOW
    analogWrite(MTBP, pb);
  }else{                      //逆転
		PORTD &= ~_BV(4);					//	PD4=LOW
		PORTD |= _BV(7);					//	PD7=HIGH
    analogWrite(MTBP, -pb);
  }
  mpowB=pb;
}

//*** Servo 操作      ************************************************
//------------------------------------------------
void	NRob3::act_ServoDir(int i, int p) //サーボ値指定駆動 i=servoNo(1or2)
{
  if(p<0)		p=0;												//リミッター
  if(p>180)	p=180;
	i--;
	svPos[i]=svTag[i]=p;													//設定

	if(i==0)		svM3.write(svPos[0]);		//servoに送る
	else				svM4.write(svPos[1]);

}
//------------------------------------------------
void	NRob3::act_ServoAbs(int i, int p)	//サーボ絶対値駆動 i=servoNo(1or2)
{
  if(p<0)		p=0;												//リミッター
  if(p>180)	p=180;
	
	svTag[i-1]=p;													//設定
	flgServoAct=true;
}
//------------------------------------------------
void	NRob3::servo_ON(void)							//サーボON
{
  svM3.attach(SVPIN_M3);        				//サーボセット
  svM4.attach(SVPIN_M4);
}
//------------------------------------------------
void	NRob3::servo_OFF(void)						//サーボOFF
{
  svM3.detach();
  svM4.detach();
}
//------------------------------------------------
int  NRob3::get_ServoPos(int no)				//サーボの現在位置を返す
{
	if(no==1)	return svPos[0];
	else			return svPos[1];
}
//------------------------------------------------
void	NRob3::act_ServoLoop(void)				//サーボloop動作
{
	bool	chk=false;
	int		d;

	for(int i=0; i<2 ;i++){								//変化あれば動作
		if(svPos[i]!=svTag[i]){
			chk = true;												//動作中フラグセット
			if(svPos[i]<svTag[i]){						//アップ
				d = svTag[i]-svPos[i];
				if(d>5){												//差が大きいときは早く
					svPos[i]+=(d/2);
				}else{
					svPos[i]=svTag[i];
				}
			}else{														//ダウン
				d = svPos[i]-svTag[i];
				if(d>5){												//差が大きいときは早く
					svPos[i]-=(d/2);
				}else{
					svPos[i]=svTag[i];
				}
			}
			if(i==0)		svM3.write(svPos[0]);		//servoに送る
			else				svM4.write(svPos[1]);
		}
	}
	flgServoAct = chk;
}

//*** 超音波センサ  *****************************************

//------------------------------------------------
void	NRob3::chk_US(void) 							//超音波センサ読込
{
  int tr, ec, dis;
	byte	no;
	static byte usno=1;

	if(isSonar){
		if(usno==1){												//select
			no=0;
			tr=S7TRIG;	ec=S7ECHO;
			if(isSonar>1)	usno=2;
		}else{
			no=1;
			tr=S8TRIG;	ec=S8ECHO;
			usno=1;
		}

  	digitalWrite(tr, HIGH);         //トリガにパルスを送る
  	delayMicroseconds(10);
  	digitalWrite(tr, LOW);
  	dis = pulseIn(ec, HIGH,50000);  //パルスを調べる  MAX50ms
  	dis = ((dis/2) / 29.1);         //ｃｍに換算
		us[no]=dis;
	}
}
//------------------------------------------------
int		NRob3::get_US(int no)
{
	if(no<3)	return us[no-1];
	else			return 0;
}
//------------------------------------------------
void	NRob3::use_US(byte num) 			//超音波センサ使用 1=1ケ/2=2ケ/0=無し
{
	isSonar = num;
}

//*** コンパスセンサ  ************************************************

//------------------------------------------------
bool  NRob3::chk_Compass(void)					//コンパスの接続を確認
{
  byte er, address, i;
	bool ans=false;
	
  for(i = 1; i < 3; i++ ){

		if(i==1)	address = ADR9250;				//9250か271の2択
		else			address = ADR271;

		er=I2C_Check(address);
 
    if (er == 0){												//見つかったらセット
			typeCompass = i;
			ans = true;
			break;
		}
	}
	return ans;
}
//------------------------------------------------
void	NRob3::init_Compass(void)					//コンパスモジュール初期化
{
	int ang=0;

	if(typeCompass){
		if(typeCompass==1)	init_9250();
		else								init_271();
	}
	for(int i=0; i<5 ;i++){
  	delay(50);
		read_Compass();
		ang += get_Angle();									//相対位置算出用角度
	}
	angOrg = ang/5;

}
//------------------------------------------------
void	NRob3::read_Compass(void)					//コンパス読み込み動作
{
	if(typeCompass){
		if(typeCompass==1)	read_9250();
		else								read_271();
	}
}
//------------------------------------------------
int	NRob3::get_Angle(void)						//コンパス読み込み
{
	return angAbs;
}
//------------------------------------------------
int	NRob3::get_RelAng(void)						//相対角度読み込み
{
	return angRel;
}

//------------------------------------------------
void	NRob3::init_271(void)							//GY271初期化
{
	I2Cadr = ADR271;
	I2Cbuf[0]=0x0B;	I2Cbuf[1]=0x01;
  I2C_Send(2); 

	I2Cbuf[0]=0x20;	I2Cbuf[1]=0x40;
  I2C_Send(2); 

	I2Cbuf[0]=0x21;	I2Cbuf[1]=0x01;
  I2C_Send(2); 

	I2Cbuf[0]=0x09;	I2Cbuf[1]=0x1D;
  I2C_Send(2); 

	I2Cbuf[0]=0x02;	I2Cbuf[1]=0x00;	//continuous measurement mode
  I2C_Send(2); 
}
//------------------------------------------------
void	NRob3::read_271(void)							//コンパス読み込み
{
  int x,y;
  float th;
  int ang;
  byte  i;

	I2Cadr = ADR271;
	I2Cbuf[0]=0x00;												//register0より
	i=I2C_Recv(6);												//6バイト読む
  if(i<5) return;             					//6バイト以下は受信エラー無視
  
  mx = I2Cbuf[0];										//受信OKでxyz展開
  mx |= I2Cbuf[1]<<8;
  my = I2Cbuf[2];
  my |= I2Cbuf[3]<<8;
  mz = I2Cbuf[4];
  mz |= I2Cbuf[5]<<8;

	chg360(mx,my);												//360度換算

}
//------------------------------------------------
void	NRob3::chg360(int x, int y)				//コンパス360度換算
{
  float th;
  int ang;

  x = x - xOfset;
  y = y - yOfset;
  
  th=atan2(x, y);
  if (th < 0)     th += 2*PI ;
  if (th > 2*PI)  th -= 2*PI;
  angAbs = int(th * 57.30);        			//360度換算

  angRel = angAbs - angOrg; 			   		//相対角度も計算
  if(angRel<0)   angRel+=360;
  if(angRel>360) angRel-=360;

}
//------------------------------------------------
void  NRob3::init_9250(void)						//コンパス9250初期化
{
  byte  ans;
  
	I2Cadr = ADR9250;
	I2Cbuf[0]=0x6B;	I2Cbuf[1]=0x00;       //スリープモードを解除
  I2C_Send(2);
  delay(50);

	I2Cbuf[0]=0x75;								  			//接続チェック
	ans=I2C_Recv(1);

  if(ans){
		I2Cbuf[0]=0x6B;	I2Cbuf[1]=0x00;     //スリープモードを解除
  	I2C_Send(2);
    delay(10);
		I2Cbuf[0]=0x37;	I2Cbuf[1]=0x02;    	//磁気センサバイパスモード
  	I2C_Send(2);
    delay(10);
		I2Cadr = ADR8963;
		I2Cbuf[0]=0x0A;	I2Cbuf[1]=0x12;    	//磁気センサのAD変換開始
  	I2C_Send(2);
  }
}
//------------------------------------------------
void  NRob3::read_9250(void)						//コンパス9250読み込み
{
  int tmp;

	I2Cadr = ADR9250;     								//MPU9250 0x3bから，14バイト分
	I2Cbuf[0]=0x3b;
	tmp=I2C_Recv(14);
  if(tmp<5) return;
  ax = (int)(I2Cbuf[0]<<8| I2Cbuf[1]);	//加速度とジャイロ読み込み
  ay = (int)(I2Cbuf[2]<<8| I2Cbuf[3]);	//ここでは計算しない
  az = (int)(I2Cbuf[4]<<8| I2Cbuf[5]);

  gx = (int)(I2Cbuf[8]<<8| I2Cbuf[9]);
  gy = (int)(I2Cbuf[10]<<8| I2Cbuf[11]);
  gz = (int)(I2Cbuf[12]<<8| I2Cbuf[13]);

	I2Cadr = ADR8963;     								//読み出し確認
	I2Cbuf[0]=0x02;
	tmp=I2C_Recv(1);

	I2Cadr = ADR8963;     								//AK8963 0x03から，7バイト分
	I2Cbuf[0]=0x03;
	tmp=I2C_Recv(7);

  mx = I2Cbuf[2];												//受信OKでxyz展開
  mx |= I2Cbuf[3]<<8;
  my =  I2Cbuf[0];
  my |= I2Cbuf[1]<<8;
  mz = I2Cbuf[4];
  mz |= I2Cbuf[5]<<8;

	chg360(mx,my);												//360度換算

}
//------------------------------------------------

//*** MPC23017  SW LED LCD 操作  *************************************

//------------------------------------------------
void	NRob3::chk_SW(void)								//SWチェック
{
	I2Cadr = MCP23017_ADR;
	I2Cbuf[0]=0x12;
  I2C_Recv(1); 

  inGPAnow = I2Cbuf[0];									 //GPA読込

  for(int i=0; i<5; i++){								//SW１つずつチェック

    if(inSW[i]==0){                     //入力待ち
      if(!((inGPAnow>>i)&1)){              //押された
        if(!((inGPApre>>i)&1)){					 //２回連続
          inSW[i]=1;										 //入力セット
          tmSW[i]=0;
        }
      }
    }else if(inSW[i]>=2){               //離し待ち
      if((inGPAnow>>i)&1){                 //離された
        if((inGPApre>>i)&1){               //２回連続
          inSW[i]=0;                      //入力リセット
        }
      }
      if(++tmSW[i]>SW_LONG){
        inSW[i]=3;                        //長押し受付
        tmSW[i]=0;
      }
    }
  }
  inGPApre=inGPAnow;
}

//------------------------------------------------
bool	NRob3::in_SW(byte no)
{
  if(inSW[no]==1){
    inSW[no]=2;
		return true;
  }else{
		return false;
	}
}

//------------------------------------------------
bool	NRob3::in_LongSW(byte no)
{
  if(inSW[no]==3){
    inSW[no]=2;
		return true;
  }else{
		return false;
	}
}

//*** MPC23017  LED  操作        ************************************

void NRob3::LED_ON(byte no)							//LED ON
{
	I2Cadr = MCP23017_ADR;
	outGPA=outGPA & ~(1<<(7-no));
	LED_send();
}

void NRob3::LED_OFF(byte no)            //LED OFF
{
	I2Cadr = MCP23017_ADR;
  outGPA = outGPA | (1<<(7-no));
	LED_send();
}

void NRob3::LED_AllOFF()                //LED 全消灯
{
	I2Cadr = MCP23017_ADR;
  outGPA = 0xFF;
	LED_send();
}

void NRob3::LED_send()                	//LED送信
{
	if(outGPA!=outGPA0){									//点灯内容変化あれば
																				//送信
		I2Cbuf[0]=0x12;
		I2Cbuf[1]=outGPA;
	  I2C_Send(2);
	}
	outGPA0 = outGPA;
}

//*** MPC23017  LCD 操作        ************************************
//------------------------------------------------
void  NRob3::LCD_init(void)
{
	I2Cadr = MCP23017_ADR;
	I2Cbuf[0]=0x13;
  
  for (byte i=0; i<3; i++) {
		I2Cbuf[1]=BIT_EN|BIT_D5|BIT_D4;
  	I2C_Send(2); 
		I2Cbuf[1]=BIT_D5|BIT_D4;
  	I2C_Send(2); 
  }
	I2Cbuf[1]=BIT_EN|BIT_D5;
 	I2C_Send(2); 
	I2Cbuf[1]=BIT_D5;
 	I2C_Send(2); 

  delay(5);

  LCD_Send4(0x28, 0);     //commands FUNCTIONSET 0x20?
  delay(5);
  LCD_Send4(0x28, 0);     // ２回
  delay(5);

  displayOn = (0x04|0x08);

  LCD_Send4(displayOn, 0);
  LCD_clear();
  LCD_Send4(0x06, 0);
}

//------------------------------------------------
void  NRob3::LCD_clear(void)
{
  LCD_Send4(0x01, 0);
  delayMicroseconds(2000);
	LCD_setCursor(0, 0);
}

//------------------------------------------------
void  NRob3::LCD_print(char *w)
{
  while(*w != '\0'){  //文字列の終端まで
    LCD_write(*w);
    w++;
  }
}

//------------------------------------------------
void  NRob3::LCD_write(byte w)
{
  LCD_Send4(w, 1);
}

//------------------------------------------------
void  NRob3::LCD_setCursor(byte x, byte y)
{
  byte  buf = 0x80;

  if(y) buf |= 0x40;										//2 line
  buf += x;                   			
  LCD_Send4(buf, 0);
}

//------------------------------------------------
void  NRob3::LCD_noCursor(void)
{
  displayOn &= ~0x02;
  LCD_Send4((0x08 | displayOn), 0);
}

//------------------------------------------------
void  NRob3::LCD_cursor(void)
{
  displayOn |= 0x02;
  LCD_Send4((0x08 | displayOn), 0);
}

//------------------------------------------------
void  NRob3::LCD_noBlink(void)
{
  displayOn &= ~0x01;
  LCD_Send4((0x08 | displayOn), 0);
}

//------------------------------------------------
void  NRob3::LCD_blink(void)
{
  displayOn |= 0x01;
  LCD_Send4((0x08 | displayOn), 0);
}

//------------------------------------------------
void  NRob3::LCD_Send4(byte value, byte mode)
{
	I2Cadr = MCP23017_ADR;
	I2Cbuf[0]=0x13;
	I2Cbuf[1]=0;
  I2C_Send(2); 


  if (value & 0x10) I2Cbuf[1] |= BIT_D4;
  if (value & 0x20) I2Cbuf[1] |= BIT_D5;
  if (value & 0x40) I2Cbuf[1] |= BIT_D6;
  if (value & 0x80) I2Cbuf[1] |= BIT_D7;

  if (mode)   I2Cbuf[1] |= (BIT_RS|BIT_EN); // RS+EN
  else        I2Cbuf[1] |= BIT_EN; 					// EN

  I2C_Send(2); 

  I2Cbuf[1] &= ~(BIT_EN);
  I2C_Send(2); 

  I2Cbuf[1] = 0;

  if (value & 0x01) I2Cbuf[1] |= BIT_D4;
  if (value & 0x02) I2Cbuf[1] |= BIT_D5;
  if (value & 0x04) I2Cbuf[1] |= BIT_D6;
  if (value & 0x08) I2Cbuf[1] |= BIT_D7;

  if (mode)   I2Cbuf[1] |= (BIT_RS|BIT_EN); // RS+EN
  else        I2Cbuf[1] |= BIT_EN; // EN

  I2C_Send(2); 

  I2Cbuf[1] &= ~(BIT_EN);
  I2C_Send(2); 
}

//*** LCD表示機能関数   **********************************************
//------------------------------------------------
void  NRob3::dsp_Number2(byte x, byte y, int no)		//２桁数字表示
{
	LCD_setCursor(x, y);
 	if(no>99){
	  LCD_write('-');    					//10の桁
  	LCD_write('-');    					//1の桁
	}else{
	  LCD_write(0x30+(no/10));    //10の桁
  	LCD_write(0x30+(no%10));    //1の桁
	}
}
//------------------------------------------------
void  NRob3::dsp_NumberB(byte x, byte y, byte n)	//1バイト表示(3桁)
{
	byte n1;

	n1=n%100;

	LCD_setCursor(x, y);
  LCD_write(0x30+(n/100));    //100の桁
  LCD_write(0x30+(n1/10));    //10の桁
  LCD_write(0x30+(n1%10));    //1の桁
}
//------------------------------------------------
void  NRob3::dsp_Number4(byte x, byte y, int n)	//2バイト表示(4桁)
{
	int n1,n2;

	n1=n%1000;
	n2=n1%100;
	
	LCD_setCursor(x, y);

  LCD_write(0x30+(n/1000));   //1000の桁
  LCD_write(0x30+(n1/100));   //100の桁
  LCD_write(0x30+(n2/10));    //10の桁
  LCD_write(0x30+(n2%10));    //1の桁
}
//------------------------------------------------
//*** I2C通信   ******************************************************

byte  NRob3::I2C_Check(byte ad)
{
  byte ans;

	Wire.beginTransmission(ad);
	ans = Wire.endTransmission();
	return(ans);
}
//------------------------------------------------
void  NRob3::I2C_Send(byte num)
{
	Wire.beginTransmission(I2Cadr);
  for (byte i=0; i<num; i++) {
		Wire.write(I2Cbuf[i]);
  }
	Wire.endTransmission();
}
//------------------------------------------------
byte  NRob3::I2C_Recv(byte num)				//
{
	I2C_Send(1);
  return(I2C_Recv0(num));
}

//------------------------------------------------
byte  NRob3::I2C_Recv0(byte num)				//
{
	byte  i=0;
	
  Wire.requestFrom(I2Cadr , num);
  while(Wire.available()){    //読込
    I2Cbuf[i] = Wire.read();
    if(i<num) i++;
  }
  return(i);
}

//*** 特殊モード  ****************************************************
//------------------------------------------------
void  NRob3::Special_Mode(void)				//特殊モード
{
	while(1){
		dsp_ADmoni();							//アナログモニタ
		dsp_USmoni();							//超音波モニタ表示
		dsp_sINmoni();						//シリアル受信モニタ表示
		dsp_Compasmoni();					//コンパスモニタ表示

		set_ParaK();							//パラメータ設定画面
	}
}
//------------------------------------------------
void  NRob3::dsp_sINmoni(void)					//カラーセンサモニタ表示
{
	byte x,y;
	byte	tm=0;
	byte	pg=0;
	
	use_SubIn(1);
  LCD_clear();                    	//モニタ画面表示
	LCD_noCursor();
  LCD_print("I");
  while(1){
		if(loop()){
			if(++tm>5){												//50ms*5=250ms毎
				tm=0;
				
			  LCD_setCursor(0, 1);						//ページ番号
				if(!pg)	LCD_print("1");
				else		LCD_print("2");

				for(int i=0; i<5; i++){       		//表示
					if(i<3){
						x=i*5+2; y=0;
					}else{
						x=(i-3)*5+2; y=1;
					}
					dsp_Number4(x,y,inVar[i+(pg*5)]);
				}
    	}
			if(in_SW(0)){											//selキーでページ切替
				if(!pg){
					pg=1;
				}else{
					break;
				}
			}
		}
	}
	use_SubIn(0);
}

//------------------------------------------------
void  NRob3::dsp_ADmoni(void)						//アナログモニタ表示
{
	byte x,y;
	byte	tm=0;

  LCD_clear();                    			//モニタ画面表示
	LCD_setCursor(0,0);										//モニタ表示
	LCD_write('A');
	LCD_setCursor(0,1);
	LCD_write('D');
	in_SW(0);	//読み捨て

  while(1){
		if(loop()){
			if(++tm>5){												//50ms*5=250ms毎
				tm=0;
				
				for(int i=0; i<6; i++){       		//表示
					if(i<3){
						x=i*5+2; y=0;
					}else{
						x=(i-3)*5+2; y=1;
					}
					dsp_Number4(x,y,ave[i]);
				}
    	}
			if(in_SW(0))	break;							//selキーで終わり

		}
	}
}
//------------------------------------------------
void  NRob3::dsp_USmoni(void)						//超音波モニタ表示
{
	byte	tm=0;

	use_US(2);

  LCD_clear();                    			//モニタ画面表示
	LCD_noCursor();
  LCD_print("US1   ----  :");
  LCD_setCursor(0, 1);
  LCD_print("US2   ----  :");
  while(1){
		if(loop()){													//表示
			tm++;															//50ms*4=200ms毎
			if(tm==2){
  			dsp_Number4(6, 0, us[0]);
			}else if(tm>4){
				tm=0;
  			dsp_Number4(6, 1, us[1]);
			}
			if(in_SW(0))	break;							//selキーで終わり
		}
	}
}

//------------------------------------------------
void  NRob3::dsp_Compasmoni(void)				//コンパスモニタ表示
{
	byte	tm=0;

  LCD_clear();                    			//モニタ画面表示
	LCD_noCursor();
  LCD_print("COMPASS a ---- :");
  LCD_setCursor(0, 1);
  LCD_print("COMPASS r ---- :");
  while(1){
		if(loop()){
			tm++;															//50ms*4=200ms毎
			if(tm==2){
				dsp_Number4(10, 0, angAbs);			//表示
			}else if(tm>4){
				tm=0;
				dsp_Number4(10, 1, angRel);
			}
	  	if(in_SW(2)){           					//UpSW In?
				get_OffsetCompas();							//オフセット得る
			}
			if(in_SW(0))	break;							//selキーで終わり
		}
	}
}
//------------------------------------------------
void  NRob3::get_OffsetCompas(void)			//コンパスオフセット調整
{
	byte	tm=0;

	int	xMax,xMin,yMax,yMin,zMax,zMin;

	xMax=yMax=zMax=-32768;
	xMin=yMin=zMin=32767;

  LCD_setCursor(10,0);
  LCD_print("----");
  LCD_setCursor(10,1);
  LCD_print("----");
  while(1){
		if(loop()){
			xMax = max(xMax, mx);					//最大最小更新
			xMin = min(xMin, mx);
			yMax = max(yMax, my);
			yMin = min(yMin, my);
			zMax = max(zMax, mz);
			zMin = min(zMin, mz);
		}
		if(in_SW(1)){											//DownSwで終わり
			xOfset=(xMax+xMin)/2;
			yOfset=(yMax+yMin)/2;
			zOfset=(zMax+zMin)/2;
			write_ofset();									//メモリ
			break;
		}
	}
  LCD_setCursor(10,0);
  LCD_print("    ");
  LCD_setCursor(10,1);
  LCD_print("    ");
}

//------------------------------------------------
void  NRob3::write_ofset(void)					//オフセットメモリ
{
	EEPROM.write(1011, lowByte(xOfset));
	EEPROM.write(1012, highByte(xOfset));
	EEPROM.write(1013, lowByte(yOfset));
	EEPROM.write(1014, highByte(yOfset));
	EEPROM.write(1015, lowByte(zOfset));
	EEPROM.write(1016, highByte(zOfset));
}
//------------------------------------------------
void  NRob3::read_ofset(void)					//オフセットメモリ
{
	xOfset = EEPROM.read(1011);
	xOfset |= EEPROM.read(1012)<<8;
	yOfset = EEPROM.read(1013);
	yOfset |= EEPROM.read(1014)<<8;
	zOfset = EEPROM.read(1015);
	zOfset |= EEPROM.read(1016)<<8;
}

//*** EEPROM   定数　ｋ　*********************************************
//------------------------------------------------
void  NRob3::set_ParaK(void)						//パラメーター設定画面
{
	byte 	flgSel=0;
	byte	nok=0;
	byte	x=5;
	int		tmp;

	LCD_clear();
	LCD_print("parameter K set");
	dsp_parK(nok);
	LCD_cursor();
	LCD_setCursor(x,1);

  while(1){
		if(loop()){
		}
		if(flgSel){													//選択動作中
			if(in_SW(4)){											//Left
				if(x>5)	x--;
				flgSel=0;
			}
			if(in_SW(3)){											//Right
				if(x<7)	x++;
				flgSel=0;
			}
			if(in_SW(2)){											//Up
				tmp=parK[nok];
				if(x==5)			tmp+=100;					//カーソル位置で加算数変える
				else if(x==6)	tmp+=10;
				else					tmp++;
				if(tmp>255)	tmp=255;
				parK[nok]=tmp;
				dsp_parK(nok);
				LCD_setCursor(x,1);
			}
			if(in_SW(1)){											//Down
				tmp=parK[nok];
				if(x==5)			tmp-=100;					//カーソル位置で加算数変える
				else if(x==6)	tmp-=10;
				else					tmp--;
				if(tmp<0)			tmp=0;
				parK[nok]=tmp;
				dsp_parK(nok);
				LCD_setCursor(x,1);
			}
			if(in_SW(0)){											//Sel
				flgSel=0;
			}
			if(flgSel==0){										//変更あれば
				LCD_noBlink();
				dsp_parK(nok);
				LCD_setCursor(x,1);
			}
		}else{															//未選択時カーソル移動
			if(in_SW(4)){											//Left
				if(x>5)	x--;
				LCD_setCursor(x,1);
			}
			if(in_SW(3)){											//Right
				if(x<7)	x++;
				LCD_setCursor(x,1);
			}
			if(in_SW(2)){											//Up
				if(nok<9)		nok++;
				else				nok=0;
				dsp_parK(nok);
				LCD_setCursor(x,1);
			}
			if(in_SW(1)){											//Down
				if(nok>0)		nok--;
				else				nok=9;
				dsp_parK(nok);
				LCD_setCursor(x,1);
			}
			if(in_SW(0)){											//Sel
				flgSel=1;
				LCD_blink();
			}
		}
		if(in_LongSW(0)){										//Sel SW 長押し メモリ
			write_parK();
			LCD_clear();
			LCD_noCursor();
			LCD_noBlink();
			break;
		}
	}
}

//------------------------------------------------
void  NRob3::dsp_parK(byte no)					//Kパラメーター表示
{
	LCD_setCursor(1, 1);
	LCD_write('K');
  LCD_write(0x30+no);   								//番号1桁
	LCD_write(':');
	dsp_NumberB(5,1,parK[no]);
}
//------------------------------------------------
void  NRob3::read_parK(void)						//Kパラメーター読込
{
  for(int i=0; i<10; i++){
		parK[i] = EEPROM.read(1000+i);
	}
}
//------------------------------------------------
void  NRob3::write_parK(void)						//Kパラメーター書込み
{
  for(int i=0; i<10; i++){
		EEPROM.write((1000+i), parK[i]);
	}
}

//*** サブボード　				*******************************************
//------------------------------------------------
void	NRob3::use_SubIn(byte num)				//入力子機使用 1=あり/0=無し
{
	isSubIn = num;
}

//------------------------------------------------
void	NRob3::read_SubIn(void)						//入力子機読み込み
{
	if(isSubIn){

		I2Cadr = ADR_SUB_IN;											//センサ要求
		I2Cbuf[0]=COM_SUB_IN;
		I2Cbuf[1]=COM_SUB_IN;
		I2Cbuf[2]=SUB_NUMBER;
  	I2C_Send(3); 

		I2C_Recv0(20);												//値読み込み

  	for(int i=0; i<10; i++){							//変数セット
			int j=i*2;
    	inVar[i] = I2Cbuf[j]; 
    	inVar[i] |= I2Cbuf[j+1]<<8;
  	}
	}
}

//------------------------------------------------
int		NRob3::get_SubIn(byte n)						//入力子機値を返す
{
	if(n>9)	return 0;
	else		return inVar[n];
}
//------------------------------------------------
void	NRob3::write_SubOut(byte no, int d)				//出力子機に書き出し
{

	I2Cadr = ADR_SUB_OUT;
	I2Cbuf[0]=6;
	I2Cbuf[1]=COM_SUB_OUT;
	I2Cbuf[2]=no;
	I2Cbuf[3]=lowByte(d);
	I2Cbuf[4]=highByte(d);
	I2Cbuf[5]=0;
	I2Cbuf[6]=0;

 	I2C_Send(6); 

}
//------------------------------------------------

